@@ -1,13 +1,17 @@
 package Tk::widgets;
 use Carp;
 
+
+use vars qw($VERSION);
+$VERSION = '3.003'; # $Id: //depot/Tk8/Tk/widgets.pm#3$
+
 sub import
 {
  my $class = shift;
  foreach (@_)
   {
    local $SIG{__DIE__} = \&Carp::croak;
-   carp "$_ already loaded" if (exists $INC{"Tk/$_.pm"});
+   # carp "$_ already loaded" if (exists $INC{"Tk/$_.pm"});
    require "Tk/$_.pm";
   }
 }
